import {Point} from './UIUtils';

/**
 * 系统图相关模型
 * @author zhaoyk
 */
export class Template {
	
	diagramType:string;
	
	id:string;
	
	name:string;
	
	code:string;
	
	frame:Container;
	
	mainPipes: Array<MainPipe>;
	
	scatteredContainers: Array<Container>;
	
	static findRelatedMainPipes(con: Container, template: Template): Array<MainPipe> {
		const mps = new Array<MainPipe>();
		while(true) {
			if(con == null)
				break;
			
			for(const mp of template.mainPipes) {
				if(mp.relatedContainers && mp.relatedContainers.indexOf(con.id) >= 0){
					if(mps.indexOf(mp) < 0) {
						mps.push(mp);	
					}
				}
			}
			
			con = con.parent;
		}
		return mps;
	}
	
}

export class Comp {

	id:string;
	
	name:string;
	
	location:Point;
	
	size:Point;
	
	position: any;
	
	compType: string;
	
	parent: Container;
				
}

export class Container extends Comp {
	
	remark:string;
	
	children: Array<Comp>;

	layout: any;
	
	equipmentTypes: Array<string>;

	dataFilter: DataFilter;
	
	equipPack: EquipPack;
	
	dynGroup: DynGroup;
	
	static getEquipBoxes(con: Container, arr?: Array<Container>){
		if(!arr)
			arr = new Array<Container>();
		if(con.equipmentTypes != null && con.equipmentTypes.length > 0)
			arr.push(con);
		else if(con.children && con.children.length > 0) {
			for(const item of con.children) {
				Container.getEquipBoxes(<Container>item, arr);
			}
		}
		return arr;
	}
	
	static findSubCons(con: Container, arr?: Array<Container>) {
		if(!arr)
			arr = new Array<Container>();
		if(con.children && con.children.length > 0) {
			for(const item of con.children) {
				arr.push(<Container>item);
				Container.findSubCons(<Container>item, arr);
			}
		}
		return arr;
	}
	
}

export class DataFilter {
	
	type: number;
	
	object: string;
	
	operation: string;
	
	value: string;
	
	items: Array<DataFilter>;
	
	not: boolean;
	
}

export class EquipPack {
	
	packByType: boolean;
	
	legendId: string;
	
	packName: string;

}

export class DynGroup {

	dynSource: string;
	
	labelPosition: string;

}

export class MainPipe {
	
	id:string;
	
	name:string;
	
	remark:string;
	
	type:string;
	
	bindEquipment:boolean;
	
	path: any[];
	
	relatedContainers: Array<string>;
	
	locationPath: Array<Array<Point>>;
			
}

export class Legend {

	id:string;
	
	code:string;

	name:string;

	remark:string;

	width:number;

	height:number;

	equipmentTypes: Array<string>;

	diagramTypes: Array<string>;

	anchors: Array<Anchor>;
	
	dataFilter: DataFilter;
	
	outline: Array<Array<Point>>;

	infos: Array<string>;
	
	iconId: string;
	
	static getPointsForAnchor(legend: Legend, location: Point): Array<Point4Anchor> {
		const size = new Point(legend.width, legend.height);
		
		const points = new Array<Point4Anchor>();
		points.push(new Point4Anchor(new Point(location.x, location.y), Point4Anchor.TL));
		points.push(new Point4Anchor(new Point(location.x + size.x, location.y), Point4Anchor.TR));
		points.push(new Point4Anchor(new Point(location.x + size.x, location.y + size.y), Point4Anchor.BR));
		points.push(new Point4Anchor(new Point(location.x, location.y + size.y), Point4Anchor.BL));
		
		const part = 6;
		var len = size.x / part;
		for (var i = 1; i < part; i++) {
			points.push(new Point4Anchor(new Point(location.x + len * i, location.y), "T" + i));
			points.push(new Point4Anchor(new Point(location.x + len * i, location.y + size.y), "B" + i));
		}
		len = size.y / part;
		for (var i = 1; i < part; i++) {
			points.push(new Point4Anchor(new Point(location.x, location.y + len * i), "L" + i));
			points.push(new Point4Anchor(new Point(location.x + size.x, location.y + len * i), "R" + i));
		}
		
		if(legend.anchors) {
			for(const anchor of legend.anchors) {
				for(const p of points) {
					if(p.code == anchor.code) {
						p.anchor = anchor;
						p.selected = true;
						break;
					}
				}
			}	
		}
		
		return points;
	}
	
}

export class Anchor {
	
	code: string;
	
	name: string;
	
	type: string;
	
	acceptRelations: Array<string>

	toEquipmentTypes: Array<string>
	
	toDataFilter: DataFilter;

	attachToOutline: boolean = true;
	
	multiple: boolean;
	
	shape: string;


	xOffset: number;
	
	yOffset: number;
	
}

export class Point4Anchor {
	
	static TL = "TL";
	static TR = "TR";
	static BR = "BR";
	static BL = "BL";
	
	point: Point;
	code: string;
	selected: boolean;
	anchor: Anchor;
	
	constructor(point: Point, code: string) {
		this.point = point;
		this.code = code;
	}
	
}


export class Diagram {
	
	id:string;
	
	name: string;
	
	type: string;
	
	system: string;
	
	templateId: string;
	
	nodes: Array<DiagramNode>;
	
	lines: Array<Line>;
	
	template: Template;
	
}

export class DiagramNode extends Comp {
	
	containerId: string;
	
	layoutIndex: number;
	
	dataObject: Object;
	
}

export class EquipmentNode extends DiagramNode {
	
	dataObjectId: string;
	
	objClassCode: string;

	legendId: string;

	legend: Legend;
	
	anchorLocations: Object;
	
	label: Label;
	
}

export class Label extends Comp {
	
	content: string;
	
}

export class Line {
	
	locationPath: Array<Point>;
	
	flag: string;
	
}